@@ -17,7 +17,7 @@ def cmd_retrieval_common_opts(sp):
     sp.add_argument('msgid', nargs='?',
                     help='Message ID to process, or pipe a raw message')
     sp.add_argument('-p', '--use-project', dest='useproject', default=None,
-                    help='Use a specific project instead of guessing (linux-mm, linux-hardening, etc)')
+                    help='Use a specific project instead of default (linux-mm, linux-hardening, etc)')
     sp.add_argument('-m', '--use-local-mbox', dest='localmbox', default=None,
                     help='Instead of grabbing a thread from lore, process this mbox file (or - for stdin)')
     sp.add_argument('-C', '--no-cache', dest='nocache', action='store_true', default=False,
@@ -36,6 +36,31 @@ def cmd_mbox_common_opts(sp):
                     help='Save as maildir (avoids mbox format ambiguities)')
 
 
+def cmd_am_common_opts(sp):
+    sp.add_argument('-v', '--use-version', dest='wantver', type=int, default=None,
+                    help='Get a specific version of the patch/series')
+    sp.add_argument('-t', '--apply-cover-trailers', dest='covertrailers', action='store_true', default=False,
+                    help='Apply trailers sent to the cover letter to all patches')
+    sp.add_argument('-S', '--sloppy-trailers', dest='sloppytrailers', action='store_true', default=False,
+                    help='Apply trailers without email address match checking')
+    sp.add_argument('-T', '--no-add-trailers', dest='noaddtrailers', action='store_true', default=False,
+                    help='Do not add or sort any trailers')
+    sp.add_argument('-s', '--add-my-sob', dest='addmysob', action='store_true', default=False,
+                    help='Add your own signed-off-by to every patch')
+    sp.add_argument('-l', '--add-link', dest='addlink', action='store_true', default=False,
+                    help='Add a Link: with message-id lookup URL to every patch')
+    sp.add_argument('-P', '--cherry-pick', dest='cherrypick', default=None,
+                    help='Cherry-pick a subset of patches (e.g. "-P 1-2,4,6-", '
+                         '"-P _" to use just the msgid specified, or '
+                         '"-P *globbing*" to match on commit subject)')
+    sp.add_argument('--cc-trailers', dest='copyccs', action='store_true', default=False,
+                    help='Copy all Cc\'d addresses into Cc: trailers')
+    sp.add_argument('--no-parent', dest='noparent', action='store_true', default=False,
+                    help='Break thread at the msgid specified and ignore any parent messages')
+    sp.add_argument('--allow-unicode-control-chars', dest='allowbadchars', action='store_true', default=False,
+                    help='Allow unicode control characters (very rarely legitimate)')
+
+
 def cmd_mbox(cmdargs):
     import b4.mbox
     b4.mbox.main(cmdargs)
@@ -51,6 +76,11 @@ def cmd_am(cmdargs):
     b4.mbox.main(cmdargs)
 
 
+def cmd_shazam(cmdargs):
+    import b4.mbox
+    b4.mbox.main(cmdargs)
+
+
 def cmd_attest(cmdargs):
     import b4.attest
     if len(cmdargs.patchfile):
@@ -87,6 +117,8 @@ def cmd():
                         help='Add more debugging info to the output')
     parser.add_argument('-q', '--quiet', action='store_true', default=False,
                         help='Output critical information only')
+    parser.add_argument('-n', '--no-interactive', action='store_true', default=False,
+                        help='Do not ask any interactive questions')
 
     subparsers = parser.add_subparsers(help='sub-command help', dest='subcmd')
 
@@ -100,39 +132,40 @@ def cmd():
     # b4 am
     sp_am = subparsers.add_parser('am', help='Create an mbox file that is ready to git-am')
     cmd_mbox_common_opts(sp_am)
-    sp_am.add_argument('-v', '--use-version', dest='wantver', type=int, default=None,
-                       help='Get a specific version of the patch/series')
-    sp_am.add_argument('-t', '--apply-cover-trailers', dest='covertrailers', action='store_true', default=False,
-                       help='Apply trailers sent to the cover letter to all patches')
-    sp_am.add_argument('-S', '--sloppy-trailers', dest='sloppytrailers', action='store_true', default=False,
-                       help='Apply trailers without email address match checking')
-    sp_am.add_argument('-T', '--no-add-trailers', dest='noaddtrailers', action='store_true', default=False,
-                       help='Do not add or sort any trailers')
-    sp_am.add_argument('-s', '--add-my-sob', dest='addmysob', action='store_true', default=False,
-                       help='Add your own signed-off-by to every patch')
-    sp_am.add_argument('-l', '--add-link', dest='addlink', action='store_true', default=False,
-                       help='Add a lore.kernel.org/r/ link to every patch')
+    cmd_am_common_opts(sp_am)
     sp_am.add_argument('-Q', '--quilt-ready', dest='quiltready', action='store_true', default=False,
                        help='Save patches in a quilt-ready folder')
-    sp_am.add_argument('-P', '--cherry-pick', dest='cherrypick', default=None,
-                       help='Cherry-pick a subset of patches (e.g. "-P 1-2,4,6-", '
-                            '"-P _" to use just the msgid specified, or '
-                            '"-P *globbing*" to match on commit subject)')
     sp_am.add_argument('-g', '--guess-base', dest='guessbase', action='store_true', default=False,
                        help='Try to guess the base of the series (if not specified)')
+    sp_am.add_argument('-b', '--guess-branch', dest='guessbranch', nargs='+', action='extend', type=str, default=None,
+                       help='When guessing base, restrict to this branch (use with -g)')
+    sp_am.add_argument('--guess-lookback', dest='guessdays', type=int, default=21,
+                       help='When guessing base, go back this many days from the patch date (default: 2 weeks)')
     sp_am.add_argument('-3', '--prep-3way', dest='threeway', action='store_true', default=False,
                        help='Prepare for a 3-way merge '
                             '(tries to ensure that all index blobs exist by making a fake commit range)')
-    sp_am.add_argument('--cc-trailers', dest='copyccs', action='store_true', default=False,
-                       help='Copy all Cc\'d addresses into Cc: trailers')
     sp_am.add_argument('--no-cover', dest='nocover', action='store_true', default=False,
                        help='Do not save the cover letter (on by default when using -o -)')
     sp_am.add_argument('--no-partial-reroll', dest='nopartialreroll', action='store_true', default=False,
                        help='Do not reroll partial series when detected')
     sp_am.set_defaults(func=cmd_am)
 
+    # b4 shazam
+    sp_sh = subparsers.add_parser('shazam', help='Like b4 am, but applies the series to your tree')
+    cmd_retrieval_common_opts(sp_sh)
+    cmd_am_common_opts(sp_sh)
+    sh_g = sp_sh.add_mutually_exclusive_group()
+    sh_g.add_argument('-H', '--make-fetch-head', dest='makefetchhead', action='store_true', default=False,
+                      help='Attempt to treat series as a pull request and fetch it into FETCH_HEAD')
+    sh_g.add_argument('-M', '--merge', dest='merge', action='store_true', default=False,
+                      help='Attempt to merge series as if it were a pull request (execs git-merge)')
+    sp_sh.add_argument('--guess-lookback', dest='guessdays', type=int, default=21,
+                       help=('(use with -H or -M) When guessing base, go back this many days from the patch date '
+                             '(default: 3 weeks)'))
+    sp_sh.set_defaults(func=cmd_shazam)
+
     # b4 attest
-    sp_att = subparsers.add_parser('attest', help='Create cryptographic attestation for a set of patches')
+    sp_att = subparsers.add_parser('attest', help='(DEPRECATED) Create cryptographic attestation for a set of patches')
     sp_att.add_argument('-f', '--from', dest='sender', default=None,
                         help='OBSOLETE: this option does nothing and will be removed')
     sp_att.add_argument('-n', '--no-submit', dest='nosubmit', action='store_true', default=False,
@@ -160,6 +193,11 @@ def cmd():
                        help='Attempt to retrieve any Link: URLs (use with -e)')
     sp_pr.add_argument('-f', '--from-addr', dest='mailfrom', default=None,
                        help='Use this From: in exploded messages (use with -e)')
+    sp_pr.add_argument('-s', '--send-as-identity', dest='sendidentity', default=None,
+                       help=('Use git-send-email to send exploded series (use with -e);'
+                             'the identity must match a [sendemail "identity"] config section'))
+    sp_pr.add_argument('--dry-run', dest='dryrun', action='store_true', default=False,
+                       help='Force a --dry-run on git-send-email invocation (use with -s)')
     sp_pr.add_argument('msgid', nargs='?',
                        help='Message ID to process, or pipe a raw message')
     sp_pr.set_defaults(func=cmd_pr)
@@ -172,7 +210,7 @@ def cmd():
                        help='Write thanks files into this dir (default=.)')
     sp_ty.add_argument('-l', '--list', action='store_true', default=False,
                        help='List pull requests and patch series you have retrieved')
-    sp_ty.add_argument('-s', '--send', default=None,
+    sp_ty.add_argument('-t', '--thank-for', dest='thankfor', default=None,
                        help='Generate thankyous for specific entries from -l (e.g.: 1,3-5,7-; or "all")')
     sp_ty.add_argument('-d', '--discard', default=None,
                        help='Discard specific messages from -l (e.g.: 1,3-5,7-; or "all")')
@@ -182,6 +220,10 @@ def cmd():
                        help='The branch to check against, instead of current')
     sp_ty.add_argument('--since', default='1.week',
                        help='The --since option to use when auto-matching patches (default=1.week)')
+    sp_ty.add_argument('-S', '--send-email', action='store_true', dest='sendemail', default=False,
+                       help='Send email instead of writing out .thanks files')
+    sp_ty.add_argument('--dry-run', action='store_true', dest='dryrun', default=False,
+                       help='Print out emails instead of sending them')
     sp_ty.set_defaults(func=cmd_ty)
 
     # b4 diff
@@ -191,7 +233,7 @@ def cmd():
     sp_diff.add_argument('-g', '--gitdir', default=None,
                          help='Operate on this git tree instead of current dir')
     sp_diff.add_argument('-p', '--use-project', dest='useproject', default=None,
-                         help='Use a specific project instead of guessing (linux-mm, linux-hardening, etc)')
+                         help='Use a specific project instead of default (linux-mm, linux-hardening, etc)')
     sp_diff.add_argument('-C', '--no-cache', dest='nocache', action='store_true', default=False,
                          help='Do not use local cache')
     sp_diff.add_argument('-v', '--compare-versions', dest='wantvers', type=int, default=None, nargs='+',
